@@ -4,9 +4,8 @@ use 5.006;
 use strict;
 use warnings;
 use XML::Parser::Lite;
-use Data::Dumper;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use vars qw( $parser );
 
@@ -28,6 +27,8 @@ sub new {
 			Char	=> sub { $self->_do_char(@_); },
 			End	=> sub { $self->_end_tag(@_); },
 			Comment	=> sub { $self->_do_comment(@_); },
+			XMLDecl	=> sub { $self->_do_xmldecl(@_); },
+			Doctype	=> sub { $self->_do_doctype(@_); },
 		};
 	$self->{process_ns} = $self->{opts}->{process_ns} || 0;
 	$self->{skip_white} = $self->{opts}->{skip_white} || 0;
@@ -35,10 +36,6 @@ sub new {
 	return $self;
 }
 
-sub _do_comment {
-	print Dumper \@_;
-}
-
 sub parse {
 	my ($self, $content) = @_;
 
@@ -51,6 +48,8 @@ sub parse {
 
 	$self->{__parser}->parse($content);
 
+	$self->cleanup($root);
+
 	if ($self->{skip_white}){
 		$self->strip_white($root);
 	}
@@ -104,6 +103,41 @@ sub _end_tag {
 	pop @{$self->{tag_stack}};
 }
 
+sub _do_comment {
+	my $self = shift;
+	shift;
+
+	for my $content(@_){
+
+		my $new_tag = {
+			'type' => 'comment',
+			'content' => $content,
+		};
+
+		push @{$self->{tag_stack}->[-1]->{children}}, $new_tag;
+	}
+}
+
+sub _do_xmldecl {
+	my $self = shift;
+	shift;
+
+	push @{$self->{tag_stack}->[-1]->{children}}, {
+		'type' => 'pi',
+		'content' => shift,
+	};
+}
+
+sub _do_doctype {
+	my $self = shift;
+	shift;
+
+	push @{$self->{tag_stack}->[-1]->{children}}, {
+		'type' => 'dtd',
+		'content' => shift,
+	};
+}
+
 sub mark_namespaces {
 	my ($self, $obj) = @_;
 
@@ -222,6 +256,52 @@ sub strip_white {
 	}
 }
 
+sub cleanup {
+	my ($self, $obj) = @_;
+
+	#
+	# cleanup PIs
+	#
+
+	if ($obj->{type} eq 'pi'){
+
+		if ($obj->{content} =~ m/^(\S+)\s+(.*)\?$/s){
+
+			delete $obj->{content};
+			$obj->{target} = $1;
+			$obj->{content} = $2;
+		}
+	}
+
+
+	#
+	# cleanup DTDs
+	#
+
+	if ($obj->{type} eq 'dtd'){
+
+		if ($obj->{content} =~ m/^(\S+)\s+(.*)$/s){
+
+			delete $obj->{content};
+			$obj->{name} = $1;
+			$obj->{content} = $2;
+		}
+	}
+
+
+	#
+	# recurse
+	#
+	
+	if ($obj->{type} eq 'root' || $obj->{type} eq 'element'){
+
+		for my $child (@{$obj->{children}}){
+
+			$self->cleanup($child);
+		}
+	}
+}
+
 
 1;
 __END__
